@@ -56,11 +56,14 @@ def norm_n2(vec) -> float:
     return (np.sum(vec**2))**0.5
 
 
-def relative_squared_error(mat1, mat2) -> float:
+def relative_squared_error(mat1, mat2, refmat=None) -> float:
     assert mat1.shape == mat2.shape
-    meansq1 = mean_square(mat1)
-    meansq2 = mean_square(mat2)
-    meansq = 0.5 * (meansq1 + meansq2)
+    if refmat is None:
+        meansq1 = mean_square(mat1)
+        meansq2 = mean_square(mat2)
+        meansq = 0.5 * (meansq1 + meansq2)
+    else:
+        meansq = mean_square(refmat)
     return mean_square(mat1 - mat2) / meansq if meansq > 0 else 1e9
 
 
@@ -93,7 +96,7 @@ def normalized_entropy(mat, nbins=3) -> float:
         return scipy.stats.entropy(freqs.frequency) / math.log(nbins)
 
 
-def relative_entropy_by_cluster(mat, lineclusters, colclusters, nbins=10):
+def relative_entropy_by_cluster(mat, lineclusters, colclusters, nbins=10) :
     assert nbins > 0
     nl, nc = mat.shape
     assert nl == len(lineclusters)
@@ -723,7 +726,7 @@ def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float,
     return df_bouquet
 
 
-def generate_clusters_centroids(ndim: int, max_corr: float):
+def generate_clusters_centroids(ndim: int, max_corr: float) -> pd.DataFrame:
 
     assert (ndim > 1)
     assert (0 <= max_corr < 1)
@@ -776,7 +779,7 @@ def generate_clusterized_vectors(veclen: int,
                                  clusterminsize: int,
                                  max_inter_corr: float,
                                  min_intra_corr: float,
-                                 random_norms: bool = True):
+                                 random_norms: bool = True) -> pd.DataFrame:
 
     clustersizes = generate_integer_partition(ntot=nbvect, nbpart=nclusters, nmin=clusterminsize)
     centroids = generate_clusters_centroids(ndim=veclen, max_corr=max_inter_corr)
@@ -798,7 +801,7 @@ def generate_clusterized_vectors(veclen: int,
     return df_res
 
 
-def build_correspondence_map(df_clusters):
+def build_correspondence_map(df_clusters) -> pd.DataFrame:
 
     assert len(df_clusters.columns) >= 2
     clusters_1 = [str(x) for x in set(df_clusters.iloc[:, 0])]
@@ -821,5 +824,20 @@ def plot_correspondence_map(df_clusters, path=None):
 
     df_map = build_correspondence_map(df_clusters)
     plot_values(df_map, xticklabels=True, path=path)
+    return None
 
 
+def duplicate_data_wfold(df: pd.DataFrame, window: int) -> pd.DataFrame:
+    if window <= 0:
+        return pd.DataFrame()
+    cols = df.columns
+    nidx = len(df.index)
+    l_shifted_cols = [[f"{c}_{i}" for i in range(window)] for c in cols]
+    shifted_cols = [item for sublist in l_shifted_cols for item in sublist]
+    duplicated_df = pd.DataFrame(index=df.index, columns=shifted_cols)
+    for irow, idx in enumerate(df.index):
+        for icol, col in enumerate(cols):
+            for shift in range(window):
+                duplicated_df.loc[idx, f"{col}_{shift}"] = df.iloc[irow+shift, icol] if irow+shift < nidx else np.nan
+    return duplicated_df
+
