@@ -33,6 +33,8 @@ if __name__ == "__main__":
     pictures_path = data_path / "Pics"
     out_data_path = imputation_paper_path / "results/"
 
+    res_path = imputation_paper_path / os.getenv("results_subpath")
+
     if os.getenv("imp_do_brunet", "False").lower() == "true":
         df_brunet = pd.read_csv(data_path / os.getenv("brunet_filename"))
         df_brunet_groups = pd.read_csv(data_path / os.getenv("brunet_grp_filename"))
@@ -275,6 +277,36 @@ if __name__ == "__main__":
 
         imp_tester.output_results(out_data_path)
 
+    if os.getenv("imp_do_screeplot_tests", "False").lower() == "true":
+        # wh_name = os.getenv("imp_wh_matname")
+        # wh_mat = pd.read_csv(data_path / wh_name)
+        # wh_mat.drop(wh_mat.columns[0], axis='columns')
+        # wh_screeplot = nmf.generate_scree_plot(wh_mat, 2, 20)
+
+        nb_clusters = 10
+        min_corr = 0.9
+        max_corr = 0.10
+        h_avg_csize = 20
+        w_avg_csize = 20
+        eps = 0
+        random_norms = True
+        censor_rate = 0.05
+        generated_M = rng.generate_nmf_reconstruction(n_comp=nb_clusters,
+                                                      min_intra_corr=min_corr,
+                                                      max_inter_corr=max_corr,
+                                                      h_avg_clust_size=h_avg_csize,
+                                                      w_avg_clust_size=w_avg_csize,
+                                                      epsilon=eps,
+                                                      random_norms=random_norms)
+        utl.censor_data(mat=generated_M, p=censor_rate, inplace=True)
+        rnstr = "RandNorms" if random_norms else "ConstNorms"
+        wh_name = f"M_nc{nb_clusters}_corrmin{round(min_corr,2)}_" \
+                  f"corrmax{round(max_corr,2)}_noise{round(eps,2)}_{rnstr}_mis{censor_rate}.csv"
+        generated_M.to_csv(res_path / "random_nmf" / wh_name,
+                           float_format="%.4f")
+        df_scree_plot = nmf.generate_scree_plot(generated_M, ncmin=2, ncmax=20)
+        pd.to_csv(res_path / f"{wh_name}_screeplot.csv")
+
     if os.getenv("imp_do_ecodata_gans", "False").lower() == "true":
         # expérience où l'on soumet les résultats de l'imputation à un GAN
         # localisation du modèle GAN
